# Find libsndfile
#
# Once done, this will define:
#
# LIBSNDFILE_FOUND
# LIBSNDFILE_INCLUDE_DIRS
# LIBSNDFILE_LIBRARIES
# LIBSNDFILE_DLL

# Find the include dir

if(WIN32)
    set(PFx86 "ProgramFiles(x86)")
    if(${CMAKE_GENERATOR_PLATFORM} MATCHES "Win32")
        set(LIBSNDFILE_ROOT "$ENV{${PFx86}}/Mega-Nerd/libsndfile")
    else()
        set(LIBSNDFILE_ROOT "$ENV{ProgramFiles}/Mega-Nerd/libsndfile")
    endif()
else()
    set(LIBSNDFILE_ROOT "/usr/local" "/usr")
endif()

find_path(LIBSNDFILE_INCLUDE_DIR
NAMES
    sndfile.h sndfile.hh
PATHS
    "${LIBSNDFILE_ROOT}/include"
)

# Find the lib:
if(WIN32)
    find_library(LIBSNDFILE_LIBRARY
    NAMES
        sndfile libsndfile-1 libsndfile
    PATHS
        "${LIBSNDFILE_ROOT}/lib"
    )

    set(LIBSNDFILE_DLL
        "${LIBSNDFILE_ROOT}/bin/libsndfile-1.dll"
    )
else()
    find_library(LIBSNDFILE_LIBRARY
        NAMES
            sndfile libsndfile-1 libsndfile
        PATHS
            /usr/local/
            /usr/lib
    )
endif()

find_package(PackageHandleStandardArgs)
find_package_handle_standard_args(LibSndFile DEFAULT_MSG LIBSNDFILE_LIBRARY LIBSNDFILE_INCLUDE_DIR)

if(LIBSNDFILE_FOUND)
    set(LIBSNDFILE_LIBRARIES ${LIBSNDFILE_LIBRARY})
    set(LIBSNDFILE_INCLUDE_DIRS ${LIBSNDFILE_INCLUDE_DIR})
    set(LIBSNDFILE_DLL ${LIBSNDFILE_DLL})
endif()

mark_as_advanced(LIBSNDFILE_LIBRARY LIBSNDFILE_LIBRARIES LIBSNDFILE_INCLUDE_DIR LIBSNDFILE_INCLUDE_DIRS LIBSNDFILE_DLL)
