#include "audiobuffer.h"

#include <iostream>
#include <algorithm>

AudioBuffer::AudioBuffer(const int numChannels, const int size)
    : m_numChannels(numChannels), m_numFrames(size)
{
    m_buffer = new float*[numChannels];
    for (int chan = 0; chan < numChannels; ++chan)
    {
        m_buffer[chan] = new float[size];
    }

    this->clear();
}

AudioBuffer::~AudioBuffer()
{
    for (int chan = 0; chan < m_numChannels; ++chan)
    {
        delete[] m_buffer[chan];
    }
    delete[] m_buffer;
}


void AudioBuffer::deinterleave(float* in, float** out, const int numChannels, const int numFrames)
{
    for (int frame = 0; frame < numFrames; ++frame)
    {
        for (int chan = 0; chan < numChannels; ++chan)
        {
            out[chan][frame] = *in;
            ++in;
        }
    }
}

void AudioBuffer::interleave(float** in, float* out, const int numChannels, const int numFrames)
{
    for (int frame = 0; frame < numFrames; ++frame)
    {
        for (int chan = 0; chan < numChannels; ++chan)
        {
            *out = in[chan][frame];
            ++out;
        }
    }
}


void AudioBuffer::clear()
{
    for (int chan = 0; chan < m_numChannels; ++chan)
    {
        std::fill(m_buffer[chan], m_buffer[chan] + m_numFrames, 0);
    }
}


float** AudioBuffer::get() const
{
    return m_buffer;
}

const int AudioBuffer::size() const
{
    return m_numFrames;
}