#ifndef LINK_AUDIOBUFFER_H
#define LINK_AUDIOBUFFER_H

#include <memory>

class AudioBuffer
{
    float** m_buffer;
    int m_numChannels;
    int m_numFrames;

public:
    AudioBuffer(const int numChannels, const int size);
    ~AudioBuffer();

    // Interleaves a multidimensional array `in` of size [numChannels][numFrames] into the single dimensional array `out`
    static void interleave(float** in, float* out, const int numChannels, const int numFrames);
    // Deinterleaves a single dimensional array `in` into the multidimensional array `out` of size [numChannels][numFrames]
    static void deinterleave(float* in, float** out, const int numChannels, const int numFrames);

    // Clears all data from AudioBuffer
    void clear();
    // Returns the raw array of data
    float** get() const;
    // Returns the number of frames available per channel
    const int size() const;
};

typedef std::shared_ptr<AudioBuffer> AudioBuffer_sPtr;
typedef std::unique_ptr<AudioBuffer> AudioBuffer_uPtr;
#endif // LINK_AUDIOBUFFER_H