#include "breakpoint.h"
#include <iostream>

Breakpoint::Breakpoint()
    : m_pos(0)
{
}

Breakpoint::~Breakpoint()
{
}

void Breakpoint::dump()
{
    for (int i = 0; i < m_points.size(); ++i)
    {
        std::cout << m_points[i].t << ", " << m_points[i].v << std::endl;
    }
}

void Breakpoint::addPoint(const float t, const float v)
{
    m_points.push_back(TVPair(t, v));
    std::sort(m_points.begin(), m_points.end());
}

void Breakpoint::tick()
{
    m_pos += 1 / double(44100 - 1);
}

void Breakpoint::reset()
{
    m_pos = 0;
}

void Breakpoint::clear()
{
    m_points.clear();
}

float Breakpoint::last_value() const
{
    return m_points.rbegin()->v;
}

float Breakpoint::get_linear() const
{
    if (m_pos <= m_points.begin()->t)
        return m_points.begin()->v;
    else if (m_pos >= m_points.rbegin()->t)
        return m_points.rbegin()->v;

    auto right = std::upper_bound(m_points.begin(), m_points.end(), m_pos);
    auto left = right - 1;

    float width = right->t - left->t;

    // if on a point, return the rhs (also stops 'same-time' points)
    if (width == 0)
        return right->v;

    // interpolate - Roads, CMT, p.321
    float frac = (m_pos - left->t) / (float)width;
    float value = left->v + ((right->v - left->v)*frac);
    return value;
}
