#ifndef LINK_BREAKPOINT_H
#define LINK_BREAKPOINT_H

#include <utility>
#include <algorithm>
#include <vector>

struct TVPair
{
    float t;
    float v;

    TVPair(float time, float value) : t(time), v(value) {}

    bool operator< (const TVPair& b) const
    {
        return t < b.t;
    }
};

// to make std::upper_bound work
inline bool operator< (const TVPair& a, const double& b)
{
    return a.t < b;
}

inline bool operator< (const double& a, const TVPair& b)
{
    return a < b.t;
}

class Breakpoint
{
    std::vector<TVPair> m_points;
    double m_pos;
public:
    Breakpoint();
    ~Breakpoint();
    void dump();
    // Adds a new TVPair - which is then sorted by time
    void addPoint(const float t, const float v);
    // Returns a linear interpolation between TVPairs based on the current read position
    float get_linear() const;
    // Advances read position by one sample
    void tick();
    // Resets read position
    void reset();
    // Removes all TVPairs
    void clear();

    // Get last value
    float last_value() const;
};

#endif // LINK_BREAKPOINT_H
