#ifndef LINK_CHAIN_H
#define LINK_CHAIN_H

#include <tinyxml2.h>

#include <string>
#include <memory>

#include <sndfile.hh>
#include "audiobuffer.h"
#include "effect.h"

#include "utils.h"

struct ChainData
{
    //TODO: RE-WRITE - don't like the semantics
    EffectData effectData; // bit of a cheat, but Effects shouldn't know about Chains

    int blockSize;
    // BODY
    unsigned int bodyFrames;
    unsigned int bodyBlocks;
    unsigned int bodyFrameRem;
    // TAIL
    unsigned int tailFrames;
    unsigned int tailBlocks;
    unsigned int tailFrameRem;
};

class Chain
{
    tinyxml2::XMLElement* m_chainRoot;

    typedef std::shared_ptr<SndfileHandle> SndfilePtr;
    ChainData m_data;
    SndfilePtr m_inFile;
    SndfilePtr m_outFile;

    AudioBuffer_uPtr m_audioBuffer;
    std::unique_ptr<float> m_interBuffer;

    std::vector<EffectPtr> m_processes;

    void setupData();
    void setupProcesses();

    void calcTail();
    void processBody(const unsigned int numBlocks, const unsigned int numFrames);
    void processTail(const unsigned int numBlocks, const unsigned int numFrames);

public:
    Chain(tinyxml2::XMLElement* chainRoot, const int blockSize);
    ~Chain();

    void addProcess(tinyxml2::XMLElement* processRoot);
    void process();
    void init();
};

typedef std::shared_ptr<Chain> ChainPtr;

#endif // LINK_CHAIN_H
