#include "delay.h"

Delay::Delay(const EffectData& data)
    : Effect(data)
{
    for(int unit = 0; unit < data.numChannels; ++unit)
    {
		m_delayUnits.push_back(new DelayUnit(44100 * 10));
    }

    addsTail(true);
    addParam("delayTime", 1, 44100 * 10, 0.5 * 44100);
    addParam("feedback", 0, 0.99, 0.5);

    calcSampleDecay();
}

Delay::~Delay()
{
	for (auto& unit : m_delayUnits)
	{
		delete unit;
	}
}

void Delay::calcSampleDecay()
{
    float impulse = 1;
    float fdbk = getParam("feedback")->last_value();

    while(impulse > 0.001)
    {
        impulse *= fdbk;
        ++m_numExtraFrames;
    }

    m_numExtraFrames *= getParam("delayTime")->last_value();
}

void Delay::reinit()
{
    m_numExtraFrames = 0;
    calcSampleDecay();

    for(auto& unit : m_delayUnits)
    {
        unit->clear();
    }
}

void Delay::process(float** in, float** out, int numChannels, int numFrames)
{
    for (int frame = 0; frame < numFrames; ++frame)
    {
	auto delayTime = getParam("delayTime")->get_linear();
	auto feedback = getParam("feedback")->get_linear();

        for(int channel = 0; channel < numChannels; ++channel)
        {
            auto& delayUnit = m_delayUnits.at(channel);

            delayUnit->write(in[channel][frame]);
            out[channel][frame] = in[channel][frame] + delayUnit->read(delayTime) * feedback;

            delayUnit->write(out[channel][frame]);
            delayUnit->tick();
        }
    
        for(auto& param : getParamList())
        {
                param->tick();
        }
    }
}
