#include "effectfactory.h"
#include "effects.h"

// Singleton factory for creating effects

EffectFactory& EffectFactory::instance()
{
    static EffectFactory instance;
    return instance;
}

EffectFactory::EffectFactory()
{
    m_effectDictionary["Gain"] = 1;
    m_effectDictionary["Delay"] = 2;
    m_effectDictionary["Split"] = 3;
    m_effectDictionary["Lowpass"] = 4;
    m_effectDictionary["Highpass"] = 5;
    m_effectDictionary["Normalise"] = 6;
	m_effectDictionary["Duration"] = 7;
}


EffectPtr EffectFactory::createByName(const char* name, const EffectData& data)
{
    EffectPtr tmp;

    switch (m_effectDictionary[name])
    {
    case 1:
        tmp = std::make_shared<Gain>(data);
        break;
    case 2:
        tmp = std::make_shared<Delay>(data);
        break;
    case 3:
        tmp = std::make_shared<Split>(data);
        break;
    case 4:
        tmp = std::make_shared<Lowpass>(data);
        break;
    case 5:
        tmp = std::make_shared<Highpass>(data);
        break;
    case 6:
        tmp = std::make_shared<Normalise>(data);
        break;
	case 7:
		tmp = std::make_shared<Duration>(data);
		break;
    default:
        tmp = nullptr;
    }
    return tmp;
}