#ifndef LINK_EFFECTFACTORY_H
#define LINK_EFFECTFACTORY_H

// Singleton factory for creating effects

#include <memory>
#include <map>
#include "effect.h"

class EffectFactory
{
public:
    // Returns singleton instance of EffectFactory
    static EffectFactory& instance();
    // Returns a std::shared_ptr<Effect> based on `name` and `data` - caller is responsible for management
    EffectPtr createByName(const char* name, const EffectData& data);

private:
    EffectFactory();
    ~EffectFactory() {}
    /* DO NOT IMPLEMENT */
    EffectFactory(const EffectFactory&) = delete;
    void operator= (EffectFactory const&) = delete;

    std::map< std::string, int > m_effectDictionary;
};

#endif // LINK_EFFECTFACTORY_H
