#include "engine.h"
#include "effectfactory.h"
#include "exceptionfactory.h"

#include <string>
#include <algorithm>
#include <iostream>

Engine::Engine(const char* xmlFile)
{
    if (m_doc.LoadFile(xmlFile) == tinyxml2::XML_ERROR_FILE_NOT_FOUND)
        throw ExceptionFactory::create(EngineError::FailedToOpen_XML);

    loadFromXML(m_doc.FirstChildElement());
}

Engine::~Engine()
{
    // RAII - everything is either on the stack, or a smart pointer
}

void Engine::loadFromXML(tinyxml2::XMLElement* rootNode)
{
    if (rootNode != nullptr)
    {
        if (strcmp(rootNode->Name(), "LINK") == 0)
        {
            tinyxml2::XMLElement* node = rootNode->FirstChildElement();
            while (node != nullptr)
            {
                if (strcmp(node->Name(), "CHAIN") == 0)
                {
                    m_chains.push_back(ChainPtr(new Chain(node, BLOCKSIZE)));
                }
                else if (strcmp(node->Name(), "MIX") == 0)
                {
                    // make a mix
                }
                else
                {
                    // unknown - ignore?
                }
                node = node->NextSiblingElement();
            }
        }
        else
        {
            throw ExceptionFactory::create(EngineError::Element_LINK_MISSING);
        }
    }
    else
    {
        throw ExceptionFactory::create(EngineError::Element_ROOT_MISSING);
    }
}


void Engine::run()
{
    int i = 1;
    for (auto& chain : m_chains)
    {
        std::cout << "Initialising chain [" << i << "/" << m_chains.size() << "] " << std::endl;
        chain->init();
        std::cout << "Processing chain [" << i << "/" << m_chains.size() << "] " << std::flush;
        chain->process();
        std::cout << ". DONE\n" << std::endl;
        ++i;
    }
}