#include "highpass.h"

#include <cmath>

Highpass::Highpass(const EffectData& data)
    : Effect(data), m_dL(0), m_dR(0)
{
    m_pi = 4.0 * atan(1.0);
    addParam("freq", 20, 20000, 200);
}

Highpass::~Highpass()
{

}

void Highpass::process(float** in, float** out, const int numChannels, const int numFrames)
{

    // audio programming, p486
    for (int frame = 0; frame < numFrames; ++frame)
    {
        costh = 2.0 - cos(2.0 * m_pi * getParam("freq")->get_linear() / m_data.sampleRate);
        coef = costh - sqrt(costh*costh - 1.0);

        out[0][frame] = (float)(in[0][frame] * (1.0 - coef) - m_dL * coef);
        out[1][frame] = (float)(in[1][frame] * (1.0 - coef) - m_dR * coef);

        m_dL = out[0][frame];
        m_dR = out[1][frame];

        // if more than stereo, straight copy
        if (numChannels > 2)
        {
            for (int chan = 2; chan < numChannels; ++chan)
            {
                out[chan][frame] = in[chan][frame];
            }
        }

        for (auto& param : getParamList())
        {
            param->tick();
        }
    }
}