#include "normalise.h"

#include <cmath>

Normalise::Normalise(const EffectData& data)
    : Effect(data), m_peak(0)
{
    this->addParam("max", 0, 1, 1);
    findPeak();
}

Normalise::~Normalise()
{

}

void Normalise::findPeak()
{
    SndfileHandle sf(m_data.inFilePath, SFM_READ);
    
    // Chain will throw before this if inFilePath invalid - just in case something happens in between
    if (sf.error() != SF_ERR_NO_ERROR)
        throw EffectException("NORMALISE: failed to open file for analysis");

    const int blockSize = 512;
    const int numBlocks = sf.frames() / blockSize;
    const int frameRem = sf.frames() - (numBlocks * blockSize);

    std::unique_ptr<double> buffer(new double[blockSize * sf.channels()]);

    for (int block = 0; block < numBlocks; ++block)
    {
        sf.readf(buffer.get(), blockSize);
        for (int sample = 0; sample < blockSize*sf.channels(); ++sample)
        {
            if (fabs(buffer.get()[sample]) > m_peak)
                m_peak = fabs(buffer.get()[sample]);
        }
    }

    sf.readf(buffer.get(), frameRem);
    for (int sample = 0; sample < frameRem*sf.channels(); ++sample)
    {
        if (fabs(buffer.get()[sample]) > m_peak)
            m_peak = fabs(buffer.get()[sample]);
    }

    m_scale = getParam("max")->get_linear() / m_peak;
}

void Normalise::process(float** in, float** out, const int numChannels, const int numFrames)
{
    for (int frame = 0; frame < numFrames; ++frame)
    {
        for (int chan = 0; chan < numChannels; ++chan)
        {
            out[chan][frame] = in[chan][frame] * m_scale;
        }
    }
}