SOC_SoundObject
{
	classvar <processGroup, <outGroup;
	var <path;
	var bounds;
	var sf, <buffer, <buffers;
	var processStack;
	var <processBus, outSynth;

	*new { | pathToSf |
		^super.newCopyArgs(pathToSf).init;
	}

	init {

		processGroup = processGroup ? Group(Server.default);
		outGroup = outGroup ? Group.after(processGroup);

		buffers = [];

		bounds = [0, -1];

		sf = SoundFile(path);
		if(sf.openRead == false)
		{
			Error("Couldn't open soundfile").throw;
		};

		processBus = Bus.audio(Server.default, if(sf.numChannels > 2) { sf.numChannels } { 2 });

		processStack = List();
	}

	at { | index |
		^processStack[index];
	}

	setBounds { | start = 0, range = -1, onLoad |
		if(buffer != nil)
		{
			buffer.free;
			buffers.do(_.free);
		};

		buffer = Buffer.read(Server.default, sf.path, start, range, {onLoad.value});
		buffers = sf.numChannels.collect({ | chan | Buffer.readChannel(Server.default, sf.path, start, range, chan); });

		bounds = [start, range ? 0];
	}

	getBounds {
		^bounds;
	}

	add { | process |
		if(process.isKindOf(SOC_Process))
		{
			processStack.add(process);
		}
		{
			Error("Invalid type: must be a SOC_Process").throw;
		}
	}

	storeOn { | stream |
		stream << "SOC_SoundObject(" << path.asString.asCompileString << ")";

		stream << ".setBounds(" <<  bounds[0] << "," << bounds[1] << ")";

		processStack.do({ | process |
			stream << ".add(" << process.asCompileString << ")";
		});
	}

	gui {
		var window, layout, lv, view, sfv;

		window = Window("SOC | Sound Object");
		layout = VLayout();

		sfv = SoundFileView().gridOn_(false);
		sfv.action_({|sfv|  var sel = sfv.selections[0]; this.setBounds(sel[0], sel[1]); });
		sfv.readFileWithTask(sf,doneAction: { | v |
			v.setSelection(0, bounds ? [0, -1]);
			v.zoomSelection(0);
		});

		layout.add(sfv);

		layout.add(
			HLayout(
				lv = ListView()
				.items_(processStack.collect(_.name))
				.action_({ | v |
					if(view != nil)
					{
						view.remove;
					};

					layout.children[0].add(view = processStack[v.value].view);
				})
			)
		);

		layout.add(Button().states_([["Process"]]).action_({ this.process; }));

		if(lv.items.size > 0)
		{
			lv.valueAction_(0);
		};

		window.layout_(layout);
		window.onClose_({ this.clear;});
		window.front;
	}

	clear {
		processStack.do(_.clear);
		outSynth.free;
	}

	process {
		var processFunc = {
			processStack.do(_.clear);
			processStack.do(_.process(this));
		};

		outGroup.freeAll;
		outSynth = { processBus.ar() }.play(outGroup);

		if(buffer == nil)
		{
			this.setBounds(0, -1, {processFunc.value});
		}
		{
			processFunc.value;
		}
	}
}